@@ -164,7 +164,9 @@ def parallel_file_path(file_path, alternative_extension):
     )
 
 
-def rat_print(file_input, file_path, team_solution, old_latex=False, pdf=True):
+def rat_print(
+    file_input, file_path, team_solution, old_latex=False, pdf=True, teamonly=False
+):
     """
     Create a document with RATs for all students and all teams.
     """
@@ -176,14 +178,14 @@ def rat_print(file_input, file_path, team_solution, old_latex=False, pdf=True):
     if team_solution in t.scratchcards:
         scratch_card_id = team_solution
         team_solution = t.scratchcards[team_solution]
-        # TODO check if solution matches requirement from questionaire
+        # TODO check if solution matches requirement from questionnaire
         tell(
             "Found scratch card {} with solution {}".format(
                 scratch_card_id, team_solution.to_string()
             )
         )
     else:
-        # TODO check if solution matches requirement from questionaire
+        # TODO check if solution matches requirement from questionnaire
         team_solution = Solution.create_solution_from_string(team_solution)
 
     # TODO check if team solution has correct number
@@ -191,12 +193,16 @@ def rat_print(file_input, file_path, team_solution, old_latex=False, pdf=True):
 
     # create a solution file
     sd = SolutionDocument()
+    # remove unnecessary answers and questions if questionnaire <10 questions
+    num_questions = len(questionaire.questions)
+    team_solution.questions = team_solution.questions[:num_questions]
+    team_solution.answers = team_solution.answers[:num_questions]
     sd.create_solution_document(t.teams, t.students, questionaire, team_solution)
     solutions_file_path = os.path.join(os.path.dirname(file_path), "solutions.teampy")
     sd.store(solutions_file_path)
     tell("Write solutions into file {}.".format(solutions_file_path))
 
-    latex = questionaire.write_latex(sd, t.teams, t.students, old_latex)
+    latex = questionaire.write_latex(sd, t.teams, t.students, old_latex, teamonly)
     if pdf:
         pdf_file_name = os.path.splitext(os.path.basename(file_path))[0] + ".pdf"
         tell("Creating PDF...")
@@ -431,6 +437,8 @@ def rat_email(file_input, file_path, testonly):
                 statuses[student_id] = "error"
                 print(e)
             bar += 1
+
+        bar.finish()
     except (
         SMTPHeloError,
         SMTPAuthenticationError,
@@ -442,7 +450,6 @@ def rat_email(file_input, file_path, testonly):
                 teampy.smtp_settings["smtp"]
             )
         )
-    bar.finish()
     server.quit()
 
     # update the status
@@ -535,12 +542,18 @@ def trial(file):
 @click.option(
     "--nopdf", default=False, is_flag=True, help="Do not create PDF, just latex source."
 )
+@click.option(
+    "--teamonly",
+    default=False,
+    is_flag=True,
+    help="Only create the pages for the teams, not the individual RATs.",
+)
 @click.option(
     "--teamsolution",
     prompt="Team solution",
     help="Code of the team scratch card or team solution.",
 )
-def print_(file, teamsolution, nopdf):
+def print_(file, teamsolution, nopdf, teamonly):
     """
     Print a RAT before class.
     """
@@ -552,10 +565,20 @@ def print_(file, teamsolution, nopdf):
         if os.path.exists(file):
             if click.confirm("Found file {}. Do you want to print it?".format(file)):
                 rat_print(
-                    click.open_file(file, encoding="utf-8"), file, teamsolution, pdf=pdf
+                    click.open_file(file, encoding="utf-8"),
+                    file,
+                    teamsolution,
+                    pdf=pdf,
+                    teamonly=teamonly,
                 )
     else:
-        rat_print(click.open_file(file, encoding="utf-8"), file, teamsolution, pdf=pdf)
+        rat_print(
+            click.open_file(file, encoding="utf-8"),
+            file,
+            teamsolution,
+            pdf=pdf,
+            teamonly=teamonly,
+        )
 
 
 def rat_setup_results_file(result_file):
@@ -610,12 +633,18 @@ def email(file, testonly):
 @rat.command()
 @click.argument("file", type=click.Path(exists=True))
 @click.option(
-    "--format", type=click.Choice(["blackboard", "supermark"], case_sensitive=False)
+    "--format",
+    type=click.Choice(
+        ["blackboard", "supermark", "pdf"],
+        case_sensitive=False,
+    ),
+    required=False,
 )
 @click.option(
     "--solution",
     help="Code of the team scratch card or team solution to shuffle the answer alternatives.",
-    prompt="Solution",
+    # prompt="Solution",
+    required=False,
 )
 def export(file, format, solution):
     """
@@ -626,17 +655,36 @@ def export(file, format, solution):
     questionaire = _load_rat_file(file_input)
     if questionaire is None:
         return
-    solution = Solution.create_solution_from_string(solution)
-    if format == "blackboard":
+    if solution is None:
+        solution = Solution.create_solution_from_questionaire(
+            questionaire, shuffle_questions=False
+        )
+        print("No solution provided, so we create a random one:")
+        print("".join(solution.answers))
+    else:
+        solution = Solution.create_solution_from_string(solution)
+    # if no format is specified, export all formats
+    if format is None:
+        format = ["blackboard", "supermark", "pdf"]
+    else:
+        format = [format]
+    if "blackboard" in format:
         text = questionaire.write_blackboard(solution)
         export_file_path = os.path.join(os.path.dirname(file), "blackboard.txt")
-        with open(export_file_path, "w", encoding="utf-8") as file:
-            file.write(text)
-    elif format == "supermark":
+        with open(export_file_path, "w", encoding="utf-8") as openfile:
+            openfile.write(text)
+    if "supermark" in format:
         text = questionaire.write_supermark(solution)
         export_file_path = os.path.join(os.path.dirname(file), "rat.md")
         with open(export_file_path, "w", encoding="utf-8") as file:
             file.write(text)
+    if "pdf" in format:
+        latex = questionaire.write_pdf(solution)
+        export_file_path = os.path.join(os.path.dirname(file), "rat.pdf")
+        # empty directory for tex reasons...
+        current_dir = os.path.abspath(os.path.dirname(file))
+        pdf = build_pdf(latex, texinputs=[current_dir, ""])
+        pdf.save_to(export_file_path)
 
 
 if __name__ == "__main__":
