@@ -28,8 +28,8 @@ def tell(message, level="info"):
 
 def tex_escape(text):
     """
-        :param text: a plain text message
-        :return: the message escaped to appear correctly in LaTeX
+    :param text: a plain text message
+    :return: the message escaped to appear correctly in LaTeX
     """
     conv = {
         "æ": r"\ae{}",
@@ -80,6 +80,7 @@ class Students:
         if all(self.df.team.apply(lambda team: team.isdigit())):
             self.df["team_int"] = self.df.team.astype(int)
         if "table" in self.df.columns:
+            # TODO - [ ] ERROR when we have a table that excel reads in as float, then no gigit!
             if all(self.df.table.apply(lambda table: table.isdigit())):
                 self.df["table_int"] = self.df.table.astype(int)
 
@@ -375,11 +376,34 @@ class Questionaire:
             preamble = myfile.read()  # .replace('\n', '')
             lines.append(preamble)
         lines.append("\\subsubsection*{{RAT Test Run}}\n")
-        index = 0
+#        index = 0
         for q in self.questions:
-            index += 1
+#            index += 1
             key = random.choice(["a", "b", "c", "d"])
-            lines.append(q.write_latex(index, key))
+#            lines.append(q.write_latex(index, key))
+            lines.append(q.write_latex(q.number, key))
+        lines.append("\\end{document}")
+        return "".join(lines)
+
+    def write_pdf(self, solution):
+        lines = []
+        # add latex preamble
+        # abs_file_path = os.path.join(os.path.dirname(__file__), 'resources', 'latex_preamble.tex')
+        abs_file_path = os.path.join(os.path.dirname(__file__), "latex_preamble.tex")
+        with open(abs_file_path, "r", encoding="utf-8") as myfile:
+            preamble = myfile.read()  # .replace('\n', '')
+            lines.append(preamble)
+        # lines.append("\\subsubsection*{{RAT}}\n")
+        lines.append("\\vbox{\\textbf"
+                     + "{{{}}}".format(self.title)
+                     + "}\n")
+        if len(self.questions) > len(solution.answers):
+            raise Exception("You must provide enough solutions. There are more questions than answers!")
+
+        for q in self.questions:
+            key = solution.answers[q.number - 1]
+            #lines.append(q.write_latex(index, key))
+            lines.append(q.write_latex(q.number, key))
         lines.append("\\end{document}")
         return "".join(lines)
 
@@ -398,7 +422,12 @@ class Questionaire:
         return "\n".join(lines)
 
     def write_latex(
-        self, solution_document=None, teams=None, students=None, old_latex=False
+        self,
+        solution_document=None,
+        teams=None,
+        students=None,
+        old_latex=False,
+        teamonly=False,
     ):
         course, date = "", ""
         page_break = "\\cleardoublepage\n\\newpage\n\n"
@@ -433,16 +462,18 @@ class Questionaire:
                         team_id, solution.card_id, solution.to_string()
                     )
                 )
-
-        # list all solution for all students of each team
-        for team_id in teams.get_ids():
-            # lines.append('\\subsubsection*\{Team {}\}\n'.format(team_id))
-            lines.append("\\subsubsection*{{Team {}}}\n".format(team_id))
-            for student_id in students.get_student_ids_of_team(team_id):
-                name = tex_escape(students.get_name(student_id))
-                solution = solution_document.student_solutions[student_id].to_string()
-                lines.append("{}:{}\\\\\n".format(name, solution))
-        lines.append(page_break)
+        if not teamonly:
+            # list all solution for all students of each team
+            for team_id in teams.get_ids():
+                # lines.append('\\subsubsection*\{Team {}\}\n'.format(team_id))
+                lines.append("\\subsubsection*{{Team {}}}\n".format(team_id))
+                for student_id in students.get_student_ids_of_team(team_id):
+                    name = tex_escape(students.get_name(student_id))
+                    solution = solution_document.student_solutions[
+                        student_id
+                    ].to_string()
+                    lines.append("{}:{}\\\\\n".format(name, solution))
+            lines.append(page_break)
 
         # TODO list solution for extra students
 
@@ -462,58 +493,61 @@ class Questionaire:
         lines.append("\\thumbnewcolumn\n")
 
         # questionaire for each student
-        old_thumb = None
-        if students.assigned_to_tables():
-            for student_id in students.get_ids(sort_by="table"):
-                name = tex_escape(students.get_name(student_id))
-                team_id = students.get_team(student_id)
-                table = students.get_table(student_id)
-                if old_thumb != table:
+        if not teamonly:
+            old_thumb = None
+            if students.assigned_to_tables():
+                for student_id in students.get_ids(sort_by="table"):
+                    name = tex_escape(students.get_name(student_id))
+                    team_id = students.get_team(student_id)
+                    table = students.get_table(student_id)
+                    if old_thumb != table:
+                        lines.append(
+                            "\\addthumb{}" + "{{{}}}".format(table) + "{white}{black}\n"
+                        )
                     lines.append(
-                        "\\addthumb{}" + "{{{}}}".format(table) + "{white}{black}\n"
+                        "\\individualprefix"
+                        + "{{{}}}{{{}}}{{{}}}{{{}}}{{{}}}{{{}}}{{{}}}".format(
+                            name, student_id, team_id, table, course, self.title, date
+                        )
+                        + "\n\n"
                     )
-                lines.append(
-                    "\\individualprefix"
-                    + "{{{}}}{{{}}}{{{}}}{{{}}}{{{}}}{{{}}}{{{}}}".format(
-                        name, student_id, team_id, table, course, self.title, date
-                    )
-                    + "\n\n"
-                )
-                solution = solution_document.student_solutions[student_id]
-                # sort question according to solution for this student
-                index = 0
-                for question_number in solution.questions:
-                    index += 1
-                    question = self.questions[question_number - 1]
-                    key = solution.answers[index - 1]
-                    lines.append(question.write_latex(index, key))
-                lines.append(page_break)
-                old_thumb = table
-        else:
-            for student_id in students.get_ids(sort_by="team"):
-                name = tex_escape(students.get_name(student_id))
-                team_id = students.get_team(student_id)
-                if old_thumb != team_id:
+                    solution = solution_document.student_solutions[student_id]
+                    # sort question according to solution for this student
+                    index = 0
+                    for question_number in solution.questions:
+                        index += 1
+                        question = self.questions[question_number - 1]
+                        key = solution.answers[index - 1]
+                        lines.append(question.write_latex(index, key))
+                    lines.append(page_break)
+                    old_thumb = table
+            else:
+                for student_id in students.get_ids(sort_by="team"):
+                    name = tex_escape(students.get_name(student_id))
+                    team_id = students.get_team(student_id)
+                    if old_thumb != team_id:
+                        lines.append(
+                            "\\addthumb{}"
+                            + "{{{}}}".format(team_id)
+                            + "{white}{black}\n"
+                        )
                     lines.append(
-                        "\\addthumb{}" + "{{{}}}".format(team_id) + "{white}{black}\n"
+                        "\\individualprefix"
+                        + "{{{}}}{{{}}}{{{}}}{{{}}}{{{}}}{{{}}}{{{}}}".format(
+                            name, student_id, team_id, "", course, self.title, date
+                        )
+                        + "\n\n"
                     )
-                lines.append(
-                    "\\individualprefix"
-                    + "{{{}}}{{{}}}{{{}}}{{{}}}{{{}}}{{{}}}{{{}}}".format(
-                        name, student_id, team_id, "", course, self.title, date
-                    )
-                    + "\n\n"
-                )
-                solution = solution_document.student_solutions[student_id]
-                # sort question according to solution for this student
-                index = 0
-                for question_number in solution.questions:
-                    index += 1
-                    question = self.questions[question_number - 1]
-                    key = solution.answers[index - 1]
-                    lines.append(question.write_latex(index, key))
-                lines.append(page_break)
-                old_thumb = team_id
+                    solution = solution_document.student_solutions[student_id]
+                    # sort question according to solution for this student
+                    index = 0
+                    for question_number in solution.questions:
+                        index += 1
+                        question = self.questions[question_number - 1]
+                        key = solution.answers[index - 1]
+                        lines.append(question.write_latex(index, key))
+                    lines.append(page_break)
+                    old_thumb = team_id
         lines.append("\\end{document}")
 
         # TODO questionaire for each extra sheet
@@ -619,7 +653,7 @@ class Solution:
         self.answers = answers
 
     @staticmethod
-    def create_solution_from_questionaire(questionaire):
+    def create_solution_from_questionaire(questionaire, shuffle_questions=True):
         # TODO we should check that the solution does not contain 10 times the same letter, because that would mess up the format of the checksum
         questions = []
         answers = []
@@ -627,8 +661,9 @@ class Solution:
         for question in range(1, len(questionaire.questions) + 1):
             questions.append(question)
             answers.append(random.choice(["a", "b", "c", "d"]))
-        # shuffle the sequence of questions
-        random.shuffle(questions)
+        if shuffle_questions:
+            # shuffle the sequence of questions
+            random.shuffle(questions)
         solution = Solution(questions, answers)
         return solution
 
@@ -841,7 +876,8 @@ class ResultLine:
                 )
                 return False
         # check that checksum is correct, and that it corresponds with result string
-        if self.type is "student":
+        # if self.type is "student":
+        if self.type == "student":
             if len(self.checksum) != 4:
                 tell(
                     "File {}, line {}: The checksum for student with id {}{} is wrong.".format(
@@ -898,7 +934,19 @@ class ResultLine:
         for c in self.normalized_results.values():
             if c == "a":
                 correct_answers += 1
-        self.score = 100 * correct_answers / len(self.normalized_results)
+# original        self.score = 100 * correct_answers / len(self.normalized_results)
+        if self.type == "student":
+            self.score = 100 * correct_answers / len(self.normalized_results)
+        elif self.type == "team":
+            total_qs = len(self.normalized_results)
+            # using the checksum as the number of attempts
+            attempts = int(self.checksum)
+            # we assume each team got all the correct answers
+            # and count the "extra" attempts to penalise the score
+            # instead of counting only the ones right on the first attempt
+            # NOTE: this assumes 4 possible options 
+            total_choices = 4
+            self.score = 100 - (attempts - total_qs) * (100/total_qs) / (total_choices - 1)
 
         # print('--Results for {}'.format(self.result_id))
         # print('              {}'.format(self.result))
